package ctbrec.ui.sites.manyvids;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ctbrec.Config;
import ctbrec.Model;
import ctbrec.io.HtmlParser;
import ctbrec.io.HttpException;
import ctbrec.sites.manyvids.MVLive;
import ctbrec.sites.manyvids.MVLiveModel;
import ctbrec.ui.PaginatedScheduledService;
import javafx.concurrent.Task;
import okhttp3.Request;
import okhttp3.Response;

public class MVLiveUpdateService extends PaginatedScheduledService {

    private static final transient Logger LOG = LoggerFactory.getLogger(MVLiveUpdateService.class);
    private String url;
    private MVLive mvlive;

    public MVLiveUpdateService(String url, MVLive mvlive) {
        this.url = url;
        this.mvlive = mvlive;
    }

    @Override
    protected Task<List<Model>> createTask() {
        return new Task<List<Model>>() {
            @Override
            public List<Model> call() throws IOException {
                LOG.debug("Fetching page {}", url);
                Request request = new Request.Builder()
                        .url(url)
                        .header("User-Agent", Config.getInstance().getSettings().httpUserAgent)
                        .header("Referer", MVLive.BASE_URL)
                        .build();
                try (Response response = mvlive.getHttpClient().execute(request)) {
                    if (response.isSuccessful()) {
                        List<Model> models = new ArrayList<>();
                        String content = response.body().string();
                        Elements cards = HtmlParser.getTags(content, "div.live-room");
                        for (Element card : cards) {
                            try {
                                String cardHtml = card.html();
                                Element link = HtmlParser.getTag(cardHtml, "h5 a");
                                MVLiveModel model = (MVLiveModel) mvlive.createModel(link.text().trim());
                                model.setUrl(mvlive.getBaseUrl() + link.attr("href"));
                                Element thumb = HtmlParser.getTag(cardHtml, "a img");
                                model.setPreview(thumb.attr("src"));
                                Element status = HtmlParser.getTag(cardHtml, "div[class~=model-status]");
                                String cssClass = status.attr("class");
                                if(cssClass.contains("live")) {
                                    model.setOnlineState(Model.State.ONLINE);
                                } else if(cssClass.contains("private")) {
                                    model.setOnlineState(Model.State.PRIVATE);
                                } else {
                                    model.setOnlineState(Model.State.UNKNOWN);
                                }
                                models.add(model);
                            } catch(RuntimeException e) {
                                if(e.getMessage().contains("No element selected by")) {
                                    // ignore
                                } else {
                                    throw e;
                                }
                            }
                        }
                        return models;
                    } else {
                        throw new HttpException(response.code(), response.message());
                    }
                }
            }
        };
    }
}
